package net.pricefx.formulaengine;

import java.util.List;
import java.util.Map;

import net.pricefx.common.apibuilder.chart.ChartBuilderResult;
import net.pricefx.domain.CompensationAgreementRecord;
import net.pricefx.server.dto.calculation.ResultCompensationAgreementRecord;
import net.pricefx.server.dto.calculation.ResultMatrix;

/**
 * The RebateRecordManager API is exposed to formula groovy elements in two separate contexts:
 * 		1) in a RebateAgreement/Compensation calculation (typeCode='RBA' aka RA/typeCode='CO' aka CO), when the formula context's 'currentItem' would be an individual line item (typeCode='RBLI' aka RLI/typeCode='COLI' aka COLI)
 * 		2) in a RebateAgreement/Compensation 'read-only' evaluation (when opening an uneditable RA/CO);  currentItem is again a RLI
 * 		3) in a RebateRecordSet/CompensationRecordSet calculation, the formula context's 'currentItem' would be an individual RebateRecord (typeCode='RR/COR')
 * The API can be accessed via the 'rebateRecords' Groovy binding.
 *
 * Not all methods are implemented in both contexts:  RR/CORs can only be added in RA/CO calculations.(1).  Similarly setRank() is only effective in context 1.
 * Context 2 obviously does not allow any modification, be it adding or updating RR/CORs.
 * Context 3 only allows updaitng existing RR/CORs (ithrough the set(..) mehtods).
 */
public interface IRebateRecordManager {

	/**
	 * Add a RR/COR to the current RA/CO.lineitem with the default name and optional initial generic attribute values
	 */
	CompensationAgreementRecord add(Map<?,?>... attributeValues);

	/**
	 * Add a RR/COR to the current RA/CO.lineitem with the given name and optional initial generic attribute values
	 */
	CompensationAgreementRecord add(String recordName, Map<?,?>... attributeValues);
	
	/**
	 * Add a RR/COR in the given RR/COR-set to the current RA/CO.lineitem with the given name and optional initial generic attribute values
	 * If the RR/COR-set with the given label does not yet exist, it will be created as a side-effect.
	 */
	CompensationAgreementRecord add(String recordName, String recordSetLabel, Map<?,?>... attributeValues);
	
	/**
	 * Sets the rank on the default RR/COR.  RR/CORs with a lower rank are calculated first within a given agreement.
	 */
	boolean setRank(int rank);
	
	/**
	 * Sets the rank on the named RR/COR.  RR/CORs with a lower rank are calculated first within a given agreement.
	 */
	boolean setRank(String recordName, int rank);

	/**
	 * Get all RR/CORs linked to the the current agreement (via RR/COR --&gt; RLI/COLO --&gt; RBA/CO).
	 */
	List<CompensationAgreementRecord> getAll();
	
	/**
	 * Get all RR/CORs linked to the line item with the given lineID in the current agreement (via RR/COR --&gt; RLI/COLI).
	 */
	List<CompensationAgreementRecord> getAll(String lineId);

	/**
	 * Get the RR/COR with the default name linked to the current RA/CO.lineitem
	 */
	CompensationAgreementRecord get();

	/**
	 * Get the RR/COR with the given name linked to the current RA/CO.lineitem
	 */
	CompensationAgreementRecord get(String recordName);

	/**
	 * In the given RR/COR-set, get the RR/COR with the given name linked to the current RA/CO.lineitem
	 */
	CompensationAgreementRecord get(String recordName, String recordSetLabel);

	/**
	 * Update genetic attributes of the RR/COR with the default name (linked to the current RA/CO.lineitem)
	 */
	boolean set(Map<?,?>... attributeValues);

	/**
	 * Update genetic attributes of the RR/COR with the given name (linked to the current RA/CO.lineitem)
	 */
	boolean set(String recordName, Map<?,?>... attributeValues);
	
	/**
	 * In the given RR/COR-set, update genetic attributes of the RR/COR with the given name (linked to the current RA/CO.lineitem)
	 */
	boolean set(String recordName, String recordSetLabel, Map<?,?>... attributeValues);

	/**
	 * Get the formula element result for the RR/COR with the default name linked to the current RA/CO.lineitem
	 */
	Object getResult(String elementName);

	/**
	 * Get the formula element result for the RR/COR with the given name linked to the current RA/CO.lineitem
	 */
	Object getResult(String recordName, String elementName);

	/**
	 * In the given RR/COR-set, get the formula element result for the RR/COR with the given name linked to the current RA/CO.lineitem
	 */
	Object getResult(String recordName, String recordSetLabel, String elementName);

	/**
	 * Return a clickable link to the RR/COR with the default name, as the result of a formula element
	 */
	ResultCompensationAgreementRecord asLink();

	/**
	 * Return a clickable link to the RR/COR with the given name, as the result of a formula element
	 */
	ResultCompensationAgreementRecord asLink(String recordName);

	/**
	 * Return a clickable link to the RR/COR for the given name and RR/COR-set, as the result of a formula element
	 */
	ResultCompensationAgreementRecord asLink(String recordName, String recordSetLabel);
	/**
	 * Return a ResultMatrix listing all RR/CORs in the agreement, optionally the fields/attributes to be shown as matrix columns.
	 */
	ResultMatrix asResultMatrix(final String... attributes);

	@Deprecated
	ChartBuilderResult buildDataTableChart(String datamartName, String dateDimFieldName);
	
}