package net.pricefx.formulaengine;

import java.util.Collection;
import java.util.Map;

import com.googlecode.genericdao.search.Filter;

import net.pricefx.domain.CompensationAgreementPayoutRecord;
import net.pricefx.domain.PayoutRecord;
import net.pricefx.service.PersistenceServiceException;

/**
 * API that can be accessed via the 'payoutRecords' binding (available in the RebateRecord calculation context — i.e., when calculating a RebateRecord in a RebateRecordCalculation task), or via the 'accrualRecords' binding (available in the CompensationRecord calculation context).
 *
 * <p>
 * Example (payoutRecords):
 * <blockquote>
 * <pre>
 * def rr = rebateRecords.get("Q1")		// api.currentItem()
 * for (def i=0; i &lt; 100000; i++) {
 * 		payoutRecords.add("Accrual", "Q1-"+i, [attribute1: rr?.attribute1, attribute2: rr?.attribute2, attribute3: rr?.attribute3] )
 * 		payoutRecords.add("Summary", "Q1-"+i, [attribute1: rr?.attribute1, attribute2: rr?.attribute2, attribute3: rr?.attribute3] )
 * }
 * </pre>
 * </blockquote>
 * Payment records managed through this API are always associated with the current RebateRecord.
 * <p>
 * Example (accrualRecords):
 * <blockquote>
 * <pre>
 *          if (api.isSyntaxCheck()) return
 *
 *          def cor = api.currentItem()
 *          def compensationId = cor?.uniqueName
 *          def compensationRecordId = cor?.sourceId
 *
 *          accrualRecords.add("Accrual", compensationId+"-"+compensationRecordId, [attribute1: "A"] )
 *          accrualRecords.add("Summary", compensationId+"-"+compensationRecordId)
 * </pre>
 * </blockquote>
 */
public interface PayoutRecordManager {

	/**
	 * Retrieves the existing payout or accrual record with the given type and name, linked to the current RebateRecord/CompensationRecord.
	 * @param type The type of the record.
	 * @param name The name of the record.
	 * @return	The payout or accrual record converted to its map representation.
	 */
	CompensationAgreementPayoutRecord get(String type, String name);

	/**
	 * Adds a new PayoutRecord or AccrualRecord, specified by the type and name.
	 * @param type	A free-text type of the record.  While it can be freely set, the usual patterns is that there are few different payment record types.
	 * @param name  A free-text name of the record. Unique within the (RebateRecord/CompensationRecord,type) namespace.
	 */
	void add(String type, String name, final Map<?,?>... attributeValues) throws PersistenceServiceException;

	/**
	 * Updates the existing PayoutRecord or AccrualRecord (if any) specified by the given type and name.
	 * @param type A free-text type of the record.  While it can be freely set, the usual patterns is that there are few different payment record types.
	 * @param name A free-text name of the record you want to update.
	 * @param attributeValues	Attribute to value map (ex. {attribute1: "ABC", ...}).  If an attribute is to be set to null, it needs to be included in the map as such.
	 * @throws PersistenceServiceException	When a PayoutRecord.attribute cannot be set to the given value
	 */
	void update(String type, String name, final Map<?,?>... attributeValues) throws PersistenceServiceException;

	/**
	 * The convenience method, adds a PayoutRecord/AccrualRecord if no record with the given type and name exists. Performs an update if the record with the given type and name already exists.
	 * Important note:  in the bulk-load mode, there is no advantage from calling the addOrUpdate method because the bulk-load always
	 * performs an insert (for a new record) or replace (for an existing record), regardless of whether the record was added with add() or addOrUpdate().
	 * @param type A free-text type of the record.  While it can be freely set, the usual patterns is that there are few different payment record types.
	 * @param name A free-text name of the record.
	 * @param attributeValues	Attribute to value map (ex. {attribute1: "ABC", ...}).  If an attribute is to be set to null, it needs to be included in the map as such.
	 * @throws PersistenceServiceException	When a PayoutRecord.attribute cannot be set to the given value
	 */
	void addOrUpdate(String type, String name, final Map<?,?>... attributeValues) throws PersistenceServiceException;

	/**
	 * Retrieves PayoutRecords or AccrualRecords linked to the current RebateRecord/CompensationRecord that match one or more optional Filter criteria.
	 * @param filters
	 * @return	Collection of PayoutRecords/AccrualRecords matching the above conditions, each in its map representation.
	 */
	Collection<CompensationAgreementPayoutRecord> find(Filter... filters);

	/**
	 * Deletes PayoutRecords/AccrualRecords linked to the current RebateRecord/CompensationRecord that match one or more optional Filter criteria.
	 * @param filters
	 * @return	The number of actually deleted records.
	 */
	int delete(Filter... filters);

	/**
	 * Switches to the bulk-load mode. It is recommended to use the bulk-load mode when the volume of records to be added or updated exceeds 100k
	 * (in regular mode, 100k records are created in around 10min).<br>
	 * Important note:  When existing records are bulk-loaded, the original records are fully overwritten. This is different
	 * from the regular mode where only the specified attributes/value pairs are updated.
	 */
	void bulkLoadMode();

	/**
	 * To be called when all records have been created and/or updated.
	 * If skipped the more recent additions and updates will not be persisted in the database.
	 */
	void flush();

}
