package net.pricefx.server.dto.calculation;

import com.fasterxml.jackson.databind.ObjectMapper;
import net.pricefx.common.api.PriceFxInterface;
import net.pricefx.domain.CustomForm;

import java.util.*;

/**
 * Utility class to work with Custom Forms in Groovy code
 */
public class CustomFormApi {
    private final Object customFormService;

    public CustomFormApi(Object objectService, ObjectMapper objectMapper) {
        this.customFormService = null; //new Object(objectService, objectMapper);
//        this.customFormService.setCustomFormManager(new CustomFormManagerUnrestrictedPermissions(objectService));
    }

    /**
     * Initiates a builder object to create a new Custom Form. The operation is finished with the create() method.
     * Please notice that if 'name' is not specified, a timestamp is used.
     * <p>
     * Example
     * <blockquote>
     *     <pre>
     *        def customForm = api.customFormApi().newCustomForm()
     *          .setLabel("my custom form 1")
     *          .setTypeId(1)
     *          .setUserGroupViewDetails("viewGroup")
     *          .setUserGroupEdit("groupEdit")
     *          .create()
     *     </pre>
     * </blockquote>
     * @return this builder instance
     */
    public CustomFormBuilder newCustomForm() {
        return new CustomFormBuilder(customFormService);
    }

    /** Initiates an operation with an existing Custom Form by id.
     *  Operations can only be 'update' for now.
     *  @see net.pricefx.server.dto.calculation.CustomFormApi.OperationsApi#updateData()
     * @param id The id of an already existing Custom Form
     * @return an Operations builder instance
     */
    public OperationsApi withId(Long id) {
        return new OperationsApi(id, customFormService);
    }

    /**
     * Helper class to create Custom Forms. The class exposes several methods of the type
     * setXX() for properties of the custom form. The custom form is then created in the
     * database by invoking the method .create().
     * Note that no permissions are checked when updating the custom form.
     * Note: when creating any logic that uses this builder, use the code
     * 'if (api.isSyntaxCheck()) return' to prevent the custom form to be created when the syntax is being checked.
     * @see {@link CustomFormApi#newCustomForm()}
     */
    public static class CustomFormBuilder {
        private final Object customFormService;
        private final Map<String, Object> data = new HashMap<>();

        private CustomFormBuilder(Object customFormService) {
            this.customFormService = customFormService;
        }

        public CustomFormBuilder setName(String name) {
            this.data.put(PriceFxInterface.FIELD_UNIQUENAME, name);
            return this;
        }

        public CustomFormBuilder setLabel(String label) {
            this.data.put(PriceFxInterface.FIELD_LABEL, label);
            return this;
        }

        /**
         * Sets the Custom Form Type id. Please notice the format required.
         * @param typeId The id of the Custom Form Type, without the '.CFOT' suffix part.
         * @return this builder instance
         */
        public CustomFormBuilder setTypeId(Long typeId) {
            this.data.put(PriceFxInterface.FIELD_TYPEDID, typeId + ".CFOT");
            return this;
        }

        public CustomFormBuilder setUserGroupViewDetails(String userGroupViewDetails) {
            this.data.put(PriceFxInterface.FIELD_USERGROUPVIEWDETAILS, userGroupViewDetails);
            return this;
        }

        public CustomFormBuilder setUserGroupEdit(String userGroupEdit) {
            this.data.put(PriceFxInterface.FIELD_USERGROUPEDIT, userGroupEdit);
            return this;
        }

        /**
         * Finishes the create operation. When this is executed, the Custom Form will be created in the database.
         * @return an Object containsing the data of the created Custom Form.
         * The data that can be extracted is:
         * <ul>
         *     <li>id</li>
         *     <li>uniqueName</li>
         *     <li>label</li>
         *     <li>attributeXX (1..30)</li>
         *     <li>inputs (the specific type of the inputs can vary)</li>
         *     <li>outputs (the specific type of the outputs can vary)</li>
         * </ul>
         */
        public Object create() {
            return null; // customFormService.add(data);
        }
    }

    public static class OperationsApi {
        private final Long id;
        private final Object customFormService;

        private OperationsApi(Long id, Object customFormService) {
            this.id = id;
            this.customFormService = customFormService;
        }

        /**
         * Initiates an update operation. The operation is finished with the .doUpdate() method
         * <p>
         * Example
         * <blockquote>
         *     <pre>
         *         customFormApi.withId(customForm.id).updateData()
         *           .setLabel("after label")
         *           .setInputValue("input1000", 1001)
         *           .doUpdate()
         *     </pre>
         * </blockquote>
         * @return an Update builder instance
         */
        public UpdateApi updateData() {
            return new UpdateApi(id, customFormService);
        }
    }

    /**
     * Helper class to update Custom Forms. The class exposes several methods of the type
     * setXX() for properties of the custom form. The custom form is then updated in the
     * database by invoking the method .doUpdate().
     * Note that no permissions are checked when updating the custom form.
     * Also note: when creating any logic that uses this builder, use the code
     * 'if (api.isSyntaxCheck()) return' to prevent the custom form to be updated when the syntax is being checked.
     * @see {@link CustomFormApi#withId(Long)}
     * @see {@link OperationsApi#updateData()}
     */
    public static class UpdateApi {
        private final Long id;
        private final Map<String, Object> inputs = new HashMap<>();
        private final Object customFormService;

        private final Map<String, Object> attributes = new HashMap<>();

        private String label;

        private UpdateApi(Long id, Object customFormService) {
            this.id = id;
            this.customFormService = customFormService;
        }

        public UpdateApi setLabel(String label) {
            this.label = label;
            return this;
        }

        /**
         * Sets the value for an input.
         * @param name the name of the input
         * @param value the value for the input. Can be a simple value, like an Integer or a String, or can be a matrix
         * value, with rows delimited by square brackets.
         * @return
         */
        public UpdateApi setInputValue(String name, Object value) {
            this.inputs.put(name, value);
            return this;
        }

        /**
         * Updates one of the 30 'attributeXX' attributes of the CustomForm class
         * @param attributeName the name of the attribute to update, for example, attribute15
         * @param value the value for that attribute
         * @return the same instance of this class
         */
        public UpdateApi setAttributeValue(String attributeName, Object value) {
            this.attributes.put(attributeName, value);
            return this;
        }

        /**
         * Finishes the update operation. When this is executed, the Custom Form will be updated in the database.
         * @return an Object containing the data of the updated Custom Form
         * The data that can be extracted is:
         * <ul>
         *     <li>id</li>
         *     <li>uniqueName</li>
         *     <li>label</li>
         *     <li>attributeXX (1..30)</li>
         *     <li>inputs (the specific type of the inputs can vary)</li>
         *     <li>outputs (the specific type of the outputs can vary)</li>
         * </ul>
         */
        public Object doUpdate() {
            return null;
        }
    }
}
