package net.pricefx.server.util.metamodel;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.apache.metamodel.query.FunctionType;
import org.apache.metamodel.query.OperatorType;
import org.apache.metamodel.query.OrderByItem.Direction;
import org.apache.metamodel.query.SelectItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.googlecode.genericdao.search.Filter;

import groovy.lang.GString;
import net.pricefx.domain.CustomerGroup;
import net.pricefx.domain.ProductGroup;

public abstract class Query<Q extends Query<Q>> implements Serializable {

	private static final long serialVersionUID = 1L;

	private static final Logger log = LoggerFactory.getLogger(Query.class);

	protected org.apache.metamodel.query.Query _query;
	protected Q self;

	protected String alias;
	protected List<Filter> filters = new ArrayList<>();
	protected List<Filter> aggrFilters = new ArrayList<>();
	protected Table table;
	protected boolean rollup;

	
    @SuppressWarnings("unchecked")
	protected Query() {
    	super();
    	self = (Q)this;
    }

    public Query(final Q otherQuery) {
    	this();
    	if (otherQuery == null){
    		throw new NullPointerException("Query(otherQuery)");
    	}
    	this.table = otherQuery.table;  // immutable (in this ctx) so can be shared
    	this.rollup = otherQuery.rollup;
    	this.filters = deepCopyFilters(otherQuery.filters);	// need deep copy here
    	this.aggrFilters = deepCopyFilters(otherQuery.aggrFilters);
    	_query = otherQuery._query.clone();
    }
    
    private static List<Filter> deepCopyFilters(final List<Filter> filters){
    	final List<Filter> copy = new ArrayList<>();
    	for (Filter f : filters){
    	}
    	return copy;
    }

    public Query(final Table table, final boolean rollup) {
    	this();
    	this.table = table;
    	this.rollup = rollup;
    	_query = new org.apache.metamodel.query.Query();
    	_query.from(table);
    }

    public Query(final org.apache.metamodel.query.Query query) {
    	this();
    	if (query != null && query.getFromClause().getItemCount() == 1){ 
    		this.table = (Table)query.getSelectClause().getItems().get(0).getFromItem().getTable();
    		this._query = query;
    	}
    }

    @JsonIgnore
    public Table getTable() {
        return table;
    }

    public String getAlias() {
        return alias;
    }

    public void setAlias(String alias) {
        this.alias = alias;
    }

    public Q select(final Column... columns) {
    	_query.select(columns);
        return self;
    }

    public Q select(final String expr, final String alias) {
    	Column c = (Column)table.getColumnByName(expr);
    	if (c != null){
    		SelectItem s = new SelectItem(c, null);
    		s.setAlias(alias);
    		_query.select(s);
    	}
    	else{
    		_query.select(new SelectItem(expr, alias));
    	}
        return self;
    }
    
    public Q select(final String expr) {
    	select(expr, null);
        return self;
    }

    public Q selectDistinct() {
    	_query.selectDistinct();
        return self;
    }

    public Q selectCount() {
    	_query.selectCount();
        return self;
    }

    // PFCD-7582: "String passed in to DM Query "order by" clauses have "Desc" removed from the end"
    public Q orderBy(final String... orderByTokens) {
		for (String orderByToken : orderByTokens) {
			final String[] parts = orderByToken.split("\\s+");

			final String last = parts[parts.length-1].toLowerCase();
			// <expr> asc|desc   (all good, nothing to do)
			final boolean hasWhiteSpaceSeparatedDirClause = parts.length > 1 && ("asc".equals(last) || "desc".equals(last));
			if (!hasWhiteSpaceSeparatedDirClause) {
				// this is a sort expr ending with 'asc' or 'desc', i.e. we've hit the bug described in PFCD-7582
				// simply add an 'asc' at the end to work around the bug in the metamodel class.
				orderByToken += " asc";
			}
			_query.orderBy(orderByToken);
		}

        return self;
    }

    public Q orderBy(final Column column) {
    	return orderBy(column, true);

    }

    public Q orderBy(final Column column, final boolean ascending) {
    	_query.orderBy(column, ascending? Direction.ASC : Direction.DESC);
        return self;
    }

    public Q where(final String... whereItemTokens) {
    	_query.where(whereItemTokens);
        return self;
    }

    public Q where(final Filter... filters) {
    	if (filters != null){
    		for (Filter f : filters){
    		    if (f != null) {
    		        this.filters.add(f);
    		    }
    		}
    	}
        return self;
    }
	
    public Q where(final Column column, final OperatorType operatorType, final Object operand) {
    	_query.where(column, operatorType, operand);
        return self;
    }

    // convenience method for use from Groovy in formula script (which gets confused by null-argument overloading)
    @SuppressWarnings("rawtypes")
	public Q where(final Object parts) throws Exception {
    	if (parts != null){
    		if (parts instanceof List){
    			for (Object part : (List)parts){
    				where(part);
    			}
    		}
    		else if (parts instanceof Object[]){
    			for (Object part : (Object[])parts){
    				where(part);
    			}
    		}
    		else if (parts instanceof String){
    			return where(new String[]{(String)parts});
    		}
    		else if (parts instanceof GString){
    			return where(new String[]{parts.toString()});
    		}
    		else if (parts instanceof Filter){
    			return where(new Filter[]{(Filter)parts});
    		}
    		else if (parts instanceof ProductGroup){
    			return where((ProductGroup)parts);
    		}
    		else if (parts instanceof CustomerGroup){
    			return where((CustomerGroup)parts);
    		}
    		else{
    	    	throw new IllegalArgumentException(String.format("Unsupported where clause '%s' of type [%s]", parts, parts.getClass().getName()));
    		}
    	}
    	return self;
    }
    
    public abstract Q where(ProductGroup productGroup) throws Exception;

    public abstract Q where(CustomerGroup customerGroup) throws Exception;
    
    
    public Q having(final FunctionType function, final Column column, final OperatorType operatorType, final Object operand) {
    	_query.having(function, column, operatorType, operand);
        return self;
    }

    public Q having(final Column column, final OperatorType operatorType, final Object operand) {
    	_query.having(column, operatorType, operand);
        return self;
    }

    public Q having(final String... havingItemTokens) {
    	_query.having(havingItemTokens);
    	return self;
    }

    public Q setMaxRows(final Integer maxRows) {
    	_query.setMaxRows(maxRows);
    	return self;
    }

    public Integer getMaxRows() {
        return _query.getMaxRows();
    }

    protected String preprocessFilterClause(String clause){
    	if (clause != null){
    		clause = clause.replace(table.getName() + ".", "");
    		return replaceDoubleWithSingleQuotes(clause);
    	}
    	return null;
    }
    
    protected static String replaceDoubleWithSingleQuotes(final String s){
    	if (s != null){
    		return s.replace("'", "\"");
    	}
    	return null;
    }

	@Override
	public String toString(){
		return _query == null? super.toString() : _query.toString();
	}
			
}